#!/usr/bin/env python
from ..utils import crypto_assets

import json
import sys

from clint import resources
from clint.arguments import Args
from tabulate import tabulate


def main():
    resources.init('madtech', 'madcc')
    if not resources.user.read('config.json'):
        config = dict()
        config['crypto_assets'] = dict()
        config['crypto_assets']['crypto_file'] = resources.user.path + '/crypto.txt'
        config['crypto_assets']['currency'] = 'eur'
        config['crypto_assets']['currency_api'] = 'http://data.fixer.io/latest?access_key=S0m3k3y'

        configfile = resources.user.open('config.json', 'w')
        configfile.write(json.dumps(config, sort_keys=True, indent=4))
    else:
        configfile = resources.user.open('config.json', 'r')
        config = json.loads(configfile.read())

    args = Args()

    if next(iter(args.grouped.get('--currency', [])), '').upper() in crypto_assets.CURRENCIES:
        currency = next(iter(args.grouped.get('--currency', [])), '')
    elif str(args.last or '').upper() in crypto_assets.CURRENCIES:
        currency = args.last
    else:
        currency = config['crypto_assets']['currency'].lower()

    if currency == 'btc':
        decimals = 10
    else:
        decimals = 2

    crypto_data = crypto_assets.parse_crypto_file(config['crypto_assets']['crypto_file'])
    if not crypto_data:
        return False

    headers, crypto_table = crypto_assets.generate_crypto_table(
        currency,
        crypto_data,
        config['crypto_assets']['currency_api']
    )
    return tabulate(crypto_table, headers=headers, floatfmt='.{}f'.format(decimals))


if __name__ == "__main__":  # pragma: no cover
    print(main())



from ..kraken.kraken import KrakenUtils


def main(authfile=None):
    k = KrakenUtils(authfile=authfile)
    deposit_limit = k.deposit_limit()
    withdraw_limit = k.withdraw_limit()

    return 'deposit max: {} EUR\nwithdraw max: {} BTC'.format(deposit_limit,
                                                              withdraw_limit)


if __name__ == "__main__":  # pragma: no cover
    print(main())



#!/usr/bin/env python
import re
import sys

import requests
from coinmarketcap import Market
from tabulate import tabulate

MISSING_CURRENCIES = ('BTC', 'USD')
CURRENCIES = ('AUD', 'BRL', 'CAD', 'CHF', 'CLP', 'CNY', 'CZK', 'DKK', 'EUR',
              'GBP', 'HKD', 'HUF', 'IDR', 'ILS', 'INR', 'JPY', 'KRW', 'MXN',
              'MYR', 'NOK', 'NZD', 'PHP', 'PKR', 'PLN', 'RUB', 'SEK', 'SGD',
              'THB', 'TRY', 'TWD', 'ZAR') + MISSING_CURRENCIES


def convert(symbol, amount, currency, currency_api):
    # Covert USD to EUR and vice versa
    # TODO: do something when api fails
    if symbol.upper() == currency.upper():
        return([symbol, amount, 1, amount])
    else:
        res = requests.get(
            currency_api,
            params={'base': symbol.upper(), 'symbols': currency.upper()}
        )
        res.raise_for_status()
        rate = res.json()['rates'][currency.upper()]
        return([symbol, amount, rate, amount * rate])


def parse_crypto_file(crypto_file):
    # Parse crypto note file for assets and amounts
    m = re.compile(r'%s.*?%s' % ('# cryptocurrency', '#'), re.S)
    try:
        with open(crypto_file) as f:
            crypto_data = list()
            res = m.search(f.read())
            for line in res.group(0).split('\n'):
                if line.startswith('-'):
                    crypto_data.append(line.strip('- \n').split())
        return crypto_data

    except IOError as e:
        print('Unable to open crypto_data file: {}'.format(crypto_file))
        return False


def retrieve_ticker_data(currency):
    # TODO: do something when api call fails
    coinmarketcap = Market()
    return coinmarketcap.ticker(start=0, limit=2000, convert=currency)


def generate_crypto_table(currency, crypto_data, currency_api):
    # Generate list of lists with crypto_data to display
    if not crypto_data:
        return False
    full_ticker_data = retrieve_ticker_data(currency)
    portfolio_total = 0
    headers = [
        'symbol', 'amount', '%',
        '{} price'.format(currency), '{} total'.format(currency)
    ]
    table = list()
    for line in crypto_data:
        symbol = line[0]
        amount = float(line[1])
        if symbol.upper() in ('EUR', 'USD'):
            outcome = convert(symbol, amount, currency, currency_api)
            table.append(outcome)
            portfolio_total += outcome[3]
            continue
        ticker_data = next((x for x in full_ticker_data if x['id'] == symbol))
        price = float(ticker_data['price_{}'.format(currency)])
        total = amount * price
        portfolio_total += total
        table.append([symbol, amount, price, total])

    for idx, val in enumerate(table):
        table[idx].insert(-2, round(val[3] / (portfolio_total / 100), 2))

    table.sort(key=lambda x: x[4], reverse=True)
    table.append([
        'total', None, None,
        None, portfolio_total
    ])

    return headers, table


def demo():
    currency = 'usd'
    decimals = 2

    # Just for demo, not actual data
    crypto_data = [['bitcoin', '10.5'], ['ethereum', '109.25']]

    headers, crypto_table = generate_crypto_table(currency, crypto_data)
    return tabulate(crypto_table, headers=headers, floatfmt='.{}f'.format(decimals))


if __name__ == "__main__":  # pragma: no cover
    print(demo())



from setuptools import setup, find_packages
from codecs import open
from os import path

here = path.abspath(path.dirname(__file__))

# Get the long description from the README file
with open(path.join(here, 'README.rst'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='madcc',
    version='0.1.1',
    description='madtech cryptocurrency functions',
    long_description=long_description,
    url='https://github.com/madeddie/madcc',
    author='Edwin Hermans',
    author_email='edwin@madtech.cx',
    license='MIT',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'Topic :: Utilities',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
    ],

    keywords='personal project cryptocurrency kraken gdax coinmarketcap',
    packages=find_packages(exclude=['contrib', 'docs', 'tests']),

    install_requires=[
        'clint',
        'coinmarketcap',
        'krakenex',
        'requests',
        'tabulate',
    ],
    python_requires='~=3.4',

    entry_points={
        'console_scripts': [
            'kraken_limits=madcc.entrypoints.kraken_limits:main',
            'crypto_assets=madcc.entrypoints.crypto_assets:main',
        ],
    },
    setup_requires=[
        'pytest-runner',
    ],
    tests_require=[
        'pytest',
        'mock',
        'pytest-mock',
        'pytest-cov',
    ],
)



#!/usr/bin/env python

import json
import os
import sys

import krakenex
import requests
from clint import resources


class KrakenUtils(object):
    def __init__(self, authfile=None):
        self._auth_file = authfile
        self._set_auth()
        if not self.api_key or not self.api_secret:
            print('No api key or secret found')
            sys.exit(1)
        self.api = krakenex.API(key=self.api_key, secret=self.api_secret)
        if not self.api_live():
            # TODO: use logging instead of print
            print('Kraken API failure')
            sys.exit(1)

    # def _init_auth_file(self):
    #     """Create default auth file if it does not exist."""
    #     auth_path = settings.user_dir.joinpath('kraken.auth')
    #     if not auth_path.is_file():
    #         answer = input(
    #             'Would you like to create the kraken_auth file? (yes|no) '
    #         ).lower().strip()
    #         if answer in ('y', 'yes'):
    #             api_key = input('Enter your kraken api key: ').strip()
    #             api_secret = input('Enter your kraken api secret: ').strip()
    #             with auth_path.open(mode='w') as auth_file:
    #                 auth_file.write(api_key)
    #                 auth_file.write(api_secret)
    #         else:
    #             print('Only public api calls allowed')

    def _set_auth(self):
        self.api_key = None
        self.api_secret = None
        """Read the api auth data from file"""
        if self._auth_file:
            self.api_key, self.api_secret = open(self._auth_file).read().splitlines()
        else:
            resources.init('madtech', 'madcc')
            self._auth_file = resources.user.path + '/kraken.auth'
            if resources.user.read('kraken.auth'):
                self.api_key, self.api_secret = resources.user.read('kraken.auth').splitlines()

    def api_live(self):
        res = self.api.query_public('Time')
        if res['error']:
            return False
        else:
            return True

    def deposit_limit(self, retry=5):
        for i in range(0, retry):
            try:
                res = self.api.query_private('DepositMethods', {'asset': 'ZEUR'})
            except json.decoder.JSONDecodeError as err:
                continue
            else:
                if res['error']:
                    # print(','.join(res['error']))
                    continue
                limit = res['result'][0]['limit']
                return limit

        return False

    def withdraw_limit(self, retry=5):
        for i in range(0, retry):
            try:
                res = self.api.query_private('WithdrawInfo', {'asset': 'XXBT', 'key': 'gdax', 'amount': 1})
            except json.decoder.JSONDecodeError as err:
                continue
            else:
                if res['error']:
                    # print(','.join(res['error']))
                    continue
                limit = res['result']['limit']
                return limit

        return False
